from infi.clickhouse_orm import engines

from .fields import DateField, DateTimeField, StringField, UInt64Field
from .db import CLUSTER_NAME, Model, DistributedModel


# TODO create table
class InGameLocationChanges(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = StringField()
    character_id = StringField(default='1')
    location = StringField()

    engine = engines.MergeTree(partition_key=('day',), order_by=('created_on', 'profile_id'))


class InGameLocationChangesDist(InGameLocationChanges, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


# TODO create table
class LevelInfoChanges(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = StringField()
    rank = UInt64Field()

    engine = engines.MergeTree(partition_key=('day',), order_by=('created_on', 'profile_id'))


class LevelInfoChangesDist(LevelInfoChanges, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


# TODO create table
class TutorialSteps(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = StringField()
    step = UInt64Field()
    env = StringField()
    player_country = StringField()
    platform_id = StringField()
    version = StringField()
    os_version = StringField()

    engine = engines.MergeTree(partition_key=('day',), order_by=('created_on', 'step'))


class TutorialStepsDist(TutorialSteps, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


# TODO create table
class UsersLoggedIn(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = StringField()
    name = StringField()
    rank = UInt64Field()
    league = UInt64Field(default=32)
    guild_id = UInt64Field()
    register_date = DateTimeField()
    platform_id = StringField()
    origin_id = StringField()
    user_timezone = StringField()
    user_language = StringField()

    engine = engines.MergeTree(partition_key=('day',), order_by=('created_on', 'profile_id'))


class UsersLoggedInDist(UsersLoggedIn, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)



import os
import re

from infi.clickhouse_orm import models
from infi.clickhouse_orm.fields import NullableField


camelcase_re = re.compile(r'([A-Z]+)(?=[a-z0-9])')

CLUSTER_NAME = os.environ.get('CLICKHOUSE_CLUSTER', 'anal')


def escape_none_values(cls, **kwargs):
    return {
        kwarg: escape_none_value(value, cls.get_field(kwarg))
        for kwarg, value in kwargs.items()
    }


def escape_none_value(value, field_cls):
    return field_cls.default \
        if value is None and not isinstance(field_cls, NullableField) else value


class Model(models.Model):
    def __init__(self, **kwargs):
        super(Model, self).__init__(**escape_none_values(self, **kwargs))

    def __setattr__(self, name, value):
        value = escape_none_value(value, self.get_field(name))
        super(Model, self).__setattr__(name, value)

    @classmethod
    def table_name(cls):
        def _join(match):
            word = match.group()

            if len(word) > 1:
                return ('_%s_%s' % (word[:-1], word[-1])).lower()

            return '_' + word.lower()

        return camelcase_re.sub(_join, cls.__name__).lstrip('_')

    def to_kafka_dict(self, include_readonly=True, field_names=None, exclude=None):
        fields = self.fields(writable=not include_readonly)

        if field_names is None:
            field_names = [f for f in fields]
        if exclude is None:
            exclude = []

        if field_names is not None:
            fields = [(n, f) for n, f in fields.items() if n in field_names and n not in exclude]

        data = self.__dict__
        kafka_dict = {}
        for name, field in fields:
            if hasattr(field, 'to_kafka'):
                kafka_dict[name] = field.to_kafka(data[name])
            else:
                kafka_dict[name] = data[name]
        return kafka_dict


class DistributedModel(Model, models.DistributedModel):
    pass



import pytz


def get_field_names(cls, *fields_cls):
    attr_list = {
        field_class.creation_counter: name
        for name, field_class in cls.fields().items()
    }
    return [attr_list[field_cls.creation_counter] for field_cls in fields_cls]


def change_field_class(ad_hoc_model, field_name, new_field_class):
    model_cls = ad_hoc_model.__class__
    # Set new field class to fields dicts
    if field_name in ad_hoc_model._fields:
        model_cls._fields[field_name] = new_field_class
        if field_name in ad_hoc_model._writable_fields:
            model_cls._writable_fields[field_name] = new_field_class
        # Update default value
        model_cls._defaults[field_name] = new_field_class.to_python(new_field_class.default, pytz.UTC)
        # Set new field class to model class
        setattr(model_cls, field_name, new_field_class)
        # Set attr value to attr for triggering field conversion/validation
        setattr(ad_hoc_model, field_name, getattr(ad_hoc_model, field_name))


from infi.clickhouse_orm import engines

from .fields import DateField, DateTimeField, JsonStringField, BoolStringField, StringField, UInt64Field
from .db import CLUSTER_NAME, Model, DistributedModel


class RewardApplied(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = StringField()
    reward_id = StringField()
    source = StringField()
    reward_type = StringField()
    count = UInt64Field()
    friend = StringField()
    extra = JsonStringField()
    autoclaim = BoolStringField()
    reward_created_on = StringField()
    accepted = BoolStringField()
    id = StringField()

    engine = engines.MergeTree(partition_key=('day',), order_by=('created_on', 'profile_id'))


class RewardAppliedDist(RewardApplied, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)
